@@ -232,7 +232,7 @@ def _rea(cac, n_regimes, L, excl_factor=5):
 def fluss(I, L, n_regimes, excl_factor=5, custom_iac=None):
     """
     Compute the Fast Low-cost Unipotent Semantic Segmentation (FLUSS)
-    for static data.
+    for static data (i.e., batch processing)
 
     Essentially, this is a wrapper to compute the corrected arc curve and
     regime locations.
@@ -291,7 +291,7 @@ def fluss(I, L, n_regimes, excl_factor=5, custom_iac=None):
 class floss(object):
     """
     Compute the Fast Low-cost Online Semantic Segmentation (FLOSS) for
-    streaming data.
+    streaming data
 
     Parameters
     ----------
@@ -429,6 +429,10 @@ class floss(object):
         self._n = self._T.shape[0]
         self._last_idx = self._n - self._m + 1  # Depends on the changing length of `T`
         self._n_appended = 0
+        self._T_isfinite = np.isfinite(self._T)
+        self._finite_T = self._T.copy()
+        self._finite_T[~np.isfinite(self._finite_T)] = 0.0
+        self._finite_Q = self._finite_T[-self._m :].copy()
 
         if self._custom_iac is None:  # pragma: no cover
             self._custom_iac = _iac(
@@ -481,9 +485,16 @@ class floss(object):
         This is the implementation for Fast Low-cost Online Semantic
         Segmentation (FLOSS).
         """
-        self._T[:] = np.roll(self._T, -1)
+        self._T[:-1] = self._T[1:]
+        self._T_isfinite[:-1] = self._T_isfinite[1:]
+        self._finite_T[:-1] = self._finite_T[1:]
+        self._finite_Q[:-1] = self._finite_Q[1:]
         self._T[-1] = t
-        Q = self._T[-self._m :]
+        self._T_isfinite[-1] = np.isfinite(t)
+        self._finite_T[-1] = t
+        if not np.isfinite(t):
+            self._finite_T[-1] = 0.0
+        self._finite_Q[-1] = self._finite_T[-1]
         excl_zone = int(np.ceil(self._m / 4))
         # Note that the start of the exclusion zone is relative to
         # the unchanging length of the matrix profile index
@@ -492,16 +503,23 @@ class floss(object):
         # Egress
         # Remove the first element in the matrix profile index
         # Shift mp up by one and replace the last row with new values
-        self._mp[:] = np.roll(self._mp, -1, axis=0)
+        self._mp[:-1, :] = self._mp[1:, :]
         self._mp[-1, 0] = np.inf
         self._mp[-1, 3] = self._last_idx
 
         # Ingress
         M_T, Σ_T = core.compute_mean_std(self._T, self._m)
 
-        D = core.mass(Q, self._T, M_T, Σ_T)
+        D = core.mass(self._finite_Q, self._finite_T, M_T, Σ_T)
         D[zone_start:] = np.inf
 
+        T_subseq_isfinite = np.all(
+            core.rolling_window(self._T_isfinite, self._m), axis=1
+        )
+        D[~T_subseq_isfinite] = np.inf
+        if not T_subseq_isfinite[-1]:
+            D[:] = np.inf
+
         # Update nearest neighbor for old data if any old subsequences
         # are closer to the newly arrived subsequence
         update_idx = np.argwhere(D < self._mp[:, 0]).flatten()
@@ -538,7 +556,7 @@ class floss(object):
         """
         Get the updated (right) matrix profile indices
         """
-        return self._mp[:, 3].astype(np.float)
+        return self._mp[:, 3].astype(np.int)
 
     @property
     def T_(self):
