@@ -9,7 +9,7 @@ import os
 import numpy as np
 from numba import cuda
 
-from . import core, _get_QT, config
+from . import core, config
 
 logger = logging.getLogger(__name__)
 
@@ -49,8 +49,8 @@ def _compute_and_update_PI_kernel(
         The time series or sequence for which to compute the dot product
 
     T_B : ndarray
-        The time series or sequence that contain your query subsequence
-        of interest
+        The time series or sequence that will be used to annotate T_A. For every
+        subsequence in T_A, its nearest neighbor in T_B will be recorded.
 
     m : int
         Window size
@@ -115,7 +115,6 @@ def _compute_and_update_PI_kernel(
     """
     start = cuda.grid(1)
     stride = cuda.gridsize(1)
-    threshold = 1e-10
 
     if i % 2 == 0:
         QT_out = QT_even
@@ -137,15 +136,21 @@ def _compute_and_update_PI_kernel(
         if math.isinf(M_T[j]) or math.isinf(μ_Q[i]):
             D = np.inf
         else:
-            if σ_Q[i] < threshold or Σ_T[j] < threshold:
+            if (
+                σ_Q[i] < config.STUMPY_STDDEV_THRESHOLD
+                or Σ_T[j] < config.STUMPY_STDDEV_THRESHOLD
+            ):
                 D = m
             else:
                 denom = m * σ_Q[i] * Σ_T[j]
-                if math.fabs(denom) < threshold:  # pragma nocover
-                    denom = threshold
+                if math.fabs(denom) < config.STUMPY_DENOM_THRESHOLD:  # pragma nocover
+                    denom = config.STUMPY_DENOM_THRESHOLD
                 D = abs(2 * m * (1.0 - (QT_out[j] - m * μ_Q[i] * M_T[j]) / denom))
 
-            if σ_Q[i] < threshold and Σ_T[j] < threshold:
+            if (
+                σ_Q[i] < config.STUMPY_STDDEV_THRESHOLD
+                and Σ_T[j] < config.STUMPY_STDDEV_THRESHOLD
+            ) or D < config.STUMPY_D_SQUARED_THRESHOLD:
                 D = 0
 
         if ignore_trivial:
@@ -192,8 +197,8 @@ def _gpu_stump(
         the matrix profile
 
     T_B_fname : str
-        The file name for the time series or sequence that contain your
-        query subsequences of interest
+        The file name for the time series or sequence that will be used to annotate T_A.
+        For every subsequence in T_A, its nearest neighbor in T_B will be recorded.
 
     m : int
         Window size
@@ -261,12 +266,12 @@ def _gpu_stump(
 
     See Table II, Figure 5, and Figure 6
 
-    Timeseries, T_B, will be annotated with the distance location
-    (or index) of all its subsequences in another times series, T_A.
+    Timeseries, T_A, will be annotated with the distance location
+    (or index) of all its subsequences in another times series, T_B.
 
-    Return: For every subsequence, Q, in T_B, you will get a distance
+    Return: For every subsequence, Q, in T_A, you will get a distance
     and index for the closest subsequence in T_A. Thus, the array
-    returned will have length T_B.shape[0]-m+1. Additionally, the
+    returned will have length T_A.shape[0]-m+1. Additionally, the
     left and right matrix profiles are also returned.
 
     Note: Unlike in the Table II where T_A.shape is expected to be equal
@@ -310,11 +315,9 @@ def _gpu_stump(
             device_M_T = cuda.to_device(M_T)
             device_Σ_T = cuda.to_device(Σ_T)
 
-        profile = np.empty((k, 3))  # float64
-        indices = np.empty((k, 3), dtype=np.int64)  # int64
+        profile = np.full((k, 3), np.inf)  # float64
+        indices = np.full((k, 3), -1, dtype=np.int64)  # int64
 
-        profile[:] = np.inf
-        indices[:, :] = -1
         device_profile = cuda.to_device(profile)
         device_indices = cuda.to_device(indices)
         _compute_and_update_PI_kernel[blocks_per_grid, threads_per_block](
@@ -370,7 +373,7 @@ def _gpu_stump(
 
 def gpu_stump(T_A, m, T_B=None, ignore_trivial=True, device_id=0):
     """
-    Compute the matrix profile with GPU-STOMP
+    Compute the z-normalized matrix profile with one or more GPU devices
 
     This is a convenience wrapper around the Numba `cuda.jit` `_gpu_stump` function
     which computes the matrix profile according to GPU-STOMP.
@@ -384,8 +387,9 @@ def gpu_stump(T_A, m, T_B=None, ignore_trivial=True, device_id=0):
         Window size
 
     T_B : (optional) ndarray
-        The time series or sequence that contain your query subsequences
-        of interest. Default is `None` which corresponds to a self-join.
+        The time series or sequence that will be used to annotate T_A. For every
+        subsequence in T_A, its nearest neighbor in T_B will be recorded. Default is
+        `None` which corresponds to a self-join.
 
     ignore_trivial : bool
         Set to `True` if this is a self-join. Otherwise, for AB-join, set this
@@ -412,12 +416,12 @@ def gpu_stump(T_A, m, T_B=None, ignore_trivial=True, device_id=0):
 
     See Table II, Figure 5, and Figure 6
 
-    Timeseries, T_B, will be annotated with the distance location
-    (or index) of all its subsequences in another times series, T_A.
+    Timeseries, T_A, will be annotated with the distance location
+    (or index) of all its subsequences in another times series, T_B.
 
-    Return: For every subsequence, Q, in T_B, you will get a distance
-    and index for the closest subsequence in T_A. Thus, the array
-    returned will have length T_B.shape[0]-m+1. Additionally, the
+    Return: For every subsequence, Q, in T_A, you will get a distance
+    and index for the closest subsequence in T_B. Thus, the array
+    returned will have length T_A.shape[0]-m+1. Additionally, the
     left and right matrix profiles are also returned.
 
     Note: Unlike in the Table II where T_A.shape is expected to be equal
@@ -437,12 +441,6 @@ def gpu_stump(T_A, m, T_B=None, ignore_trivial=True, device_id=0):
         T_B = T_A
         ignore_trivial = True
 
-    # Swap T_A and T_B for GPU implementation
-    # This keeps the API identical to and compatible with `stumpy.stump`
-    tmp_T = T_A
-    T_A = T_B
-    T_B = tmp_T
-
     T_A, M_T, Σ_T = core.preprocess(T_A, m)
     T_B, μ_Q, σ_Q = core.preprocess(T_B, m)
 
@@ -458,9 +456,6 @@ def gpu_stump(T_A, m, T_B=None, ignore_trivial=True, device_id=0):
             "For multidimensional STUMP use `stumpy.mstump` or `stumpy.mstumped`"
         )
 
-    core.check_dtype(T_A)
-    core.check_dtype(T_B)
-
     core.check_window_size(m)
 
     if ignore_trivial is False and core.are_arrays_equal(T_A, T_B):  # pragma: no cover
@@ -514,7 +509,7 @@ def gpu_stump(T_A, m, T_B=None, ignore_trivial=True, device_id=0):
     for idx, start in enumerate(range(0, l, step)):
         stop = min(l, start + step)
 
-        QT, QT_first = _get_QT(start, T_A, T_B, m)
+        QT, QT_first = core._get_QT(start, T_A, T_B, m)
         QT_fname = core.array_to_temp_file(QT)
         QT_first_fname = core.array_to_temp_file(QT_first)
         QT_fnames.append(QT_fname)
