@@ -216,7 +216,7 @@ def _get_first_mstump_profile(
     core.apply_exclusion_zone(D, start, excl_zone)
 
     P = np.full(d, np.inf, dtype="float64")
-    I = np.ones(d, dtype="int64") * -1
+    I = np.full(d, -1, dtype="int64")
 
     for i in range(d):
         min_index = np.argmin(D[i])
@@ -538,7 +538,12 @@ def _mstump(
 
         if include is not None:
             _apply_include(
-                D, include, restricted_indices, unrestricted_indices, mask, tmp_swap,
+                D,
+                include,
+                restricted_indices,
+                unrestricted_indices,
+                mask,
+                tmp_swap,
             )
             start_row_idx = include.shape[0]
 
@@ -554,7 +559,7 @@ def _mstump(
 
 def mstump(T, m, include=None, discords=False):
     """
-    Compute the multi-dimensional matrix profile with parallelized mSTOMP
+    Compute the multi-dimensional z-normalized matrix profile
 
     This is a convenience wrapper around the Numba JIT-compiled parallelized
     `_mstump` function which computes the multi-dimensional matrix profile and
@@ -614,9 +619,6 @@ def mstump(T, m, include=None, discords=False):
         err = f"T is {T_A.ndim}-dimensional and must be at least 1-dimensional"
         raise ValueError(f"{err}")
 
-    core.check_dtype(T_A)
-    core.check_dtype(T_B)
-
     core.check_window_size(m)
 
     if include is not None:
