@@ -5,346 +5,13 @@
 import logging
 
 import numpy as np
-from numba import njit, prange, config
+from numba import njit, config
 
-from . import core
+from . import core, _stump
 
 logger = logging.getLogger(__name__)
 
 
-@njit(fastmath=True)
-def _get_max_order_idx(m, n_A, n_B, orders, start, percentage):
-    """
-    Determine the order index for when the desired percentage of distances is computed
-
-    Parameters
-    ----------
-    m : int
-        Window size
-
-    n_A : int
-        The length of the time series or sequence for which to compute the matrix
-        profile `T_A`
-
-    n_B : int
-        The length of the time series or sequence that contain your query subsequences
-        of interest `T_B`
-
-    orders : ndarray
-        The order of diagonals to process and compute
-
-    start : int
-        The (inclusive) order index from which to start
-
-    percentage : float
-        Approximate percentage completed. The value is between 0.0 and 1.0.
-
-    Returns
-    -------
-    max_order_id : int
-        The order index that corresponds to desired percentage of distances to compute
-
-    n_dist_computed : int
-        The number of distances computed
-    """
-    max_n_dist = 0
-    for order_idx in range(orders.shape[0]):
-        k = orders[order_idx]
-        if k >= 0:
-            max_n_dist += min(n_A - m + 1 - k, n_B - m + 1)
-        else:
-            max_n_dist += min(n_A - m + 1, n_B - m + 1 + k)
-
-    n_dist_computed = 0
-    for order_idx in range(start, orders.shape[0]):
-        k = orders[order_idx]
-        if k >= 0:
-            n_dist_computed += min(n_A - m + 1 - k, n_B - m + 1)
-        else:
-            n_dist_computed += min(n_A - m + 1, n_B - m + 1 + k)
-
-        if n_dist_computed / max_n_dist > percentage:  # pragma: no cover
-            break
-
-    max_order_idx = order_idx + 1
-
-    return max_order_idx, n_dist_computed
-
-
-@njit(fastmath=True)
-def _get_orders_ranges(n_split, m, n_A, n_B, orders, start, percentage):
-    """
-    For the desired percentage of distances to be computed from orders, split the
-    orders into `n_split` chunks, and determine the appropriate start and stop
-    (exclusive) order index ranges for each chunk
-
-    Parameters
-    ----------
-    n_split : int
-        The number of chunks to split the percentage of desired orders into
-
-    m : int
-        Window size
-
-    n_A : int
-        The length of the time series or sequence for which to compute the matrix
-        profile `T_A`
-
-    n_B : int
-        The length of the time series or sequence that contain your query subsequences
-        of interest `T_B`
-
-    orders : ndarray
-        The order of diagonals to process and compute
-
-    start : int
-        The (inclusive) order index from which to start
-
-    percentage : float
-        Approximate percentage completed. The value is between 0.0 and 1.0.
-
-    Returns
-    -------
-    ranges : int
-        The start (column 1) and (exclusive) stop (column 2) orders index ranges
-        that corresponds to a desired percentage of distances to compute
-    """
-    max_order_idx, n_dist_computed = _get_max_order_idx(
-        m, n_A, n_B, orders, start, percentage
-    )
-
-    orders_ranges = np.zeros((n_split, 2), np.int64)
-    ranges_idx = 0
-    range_start_idx = start
-    max_n_dist_per_range = n_dist_computed / n_split
-
-    n_dist_computed = 0
-    for order_idx in range(start, max_order_idx):
-        k = orders[order_idx]
-        if k >= 0:
-            n_dist_computed += min(n_A - m + 1 - k, n_B - m + 1)
-        else:
-            n_dist_computed += min(n_A - m + 1, n_B - m + 1 + k)
-
-        if n_dist_computed > max_n_dist_per_range:  # pragma: no cover
-            orders_ranges[ranges_idx, 0] = range_start_idx
-            orders_ranges[ranges_idx, 1] = min(
-                order_idx + 1, max_order_idx
-            )  # Exclusive stop index
-            # Reset and Update
-            range_start_idx = order_idx + 1
-            ranges_idx += 1
-            n_dist_computed = 0
-
-    # Handle final range outside of for loop if it was not saturated
-    if ranges_idx < orders_ranges.shape[0]:
-        orders_ranges[ranges_idx, 0] = range_start_idx
-        orders_ranges[ranges_idx, 1] = max_order_idx
-
-    return orders_ranges
-
-
-@njit(fastmath=True)
-def _compute_diagonal(
-    T_A,
-    T_B,
-    m,
-    M_T,
-    Σ_T,
-    μ_Q,
-    σ_Q,
-    orders,
-    orders_start_idx,
-    orders_stop_idx,
-    thread_idx,
-    P,
-    I,
-    ignore_trivial,
-):
-    """
-    Compute (Numba JIT-compiled) and update P, I along a single diagonal using a single
-    thread and avoiding race conditions
-
-    Parameters
-    ----------
-    T_A : ndarray
-        The time series or sequence for which to compute the matrix profile
-
-    T_B : ndarray
-        The time series or sequence that contain your query subsequences
-        of interest
-
-    m : int
-        Window size
-
-    M_T : ndarray
-        Sliding mean of time series, `T_A`
-
-    Σ_T : ndarray
-        Sliding standard deviation of time series, `T_A`
-
-    μ_Q : ndarray
-        Mean of the query sequence, `Q`, relative to the current sliding window in `T_B`
-
-    σ_Q : ndarray
-        Standard deviation of the query sequence, `Q`, relative to the current
-        sliding window in `T_B`
-
-    orders : ndarray
-        The order of diagonals to process and compute
-
-    orders_start_idx : int
-        The start index for a range of diagonal order to process and compute
-
-    orders_stop_idx : int
-        The (exclusive) stop index for a range of diagonal order to process and compute
-
-    P : ndarray
-        Matrix profile
-
-    I : ndarray
-        Matrix profile indices
-
-    thread_idx : int
-        The thread index
-
-    ignore_trivial : bool
-        Set to `True` if this is a self-join. Otherwise, for AB-join, set this to
-        `False`. Default is `True`.
-
-    Returns
-    -------
-    None
-    """
-    n_A = T_A.shape[0]
-    n_B = T_B.shape[0]
-
-    for order_idx in range(orders_start_idx, orders_stop_idx):
-        k = orders[order_idx]
-
-        if k >= 0:
-            iter_range = range(0, min(n_B - m + 1, n_A - m + 1 - k))
-        else:
-            iter_range = range(-k, min(n_B - m + 1, n_A - m + 1 - k))
-
-        for i in iter_range:
-            if i == 0 or i == k or (k < 0 and i == -k):
-                QT = np.dot(T_A[i + k : i + k + m], T_B[i : i + m])
-            else:
-                QT = (
-                    QT
-                    - T_A[i + k - 1] * T_B[i - 1]
-                    + T_A[i + k + m - 1] * T_B[i + m - 1]
-                )
-
-            D_squared = core._calculate_squared_distance(
-                m, QT, M_T[i + k], Σ_T[i + k], μ_Q[i], σ_Q[i],
-            )
-
-            if D_squared < P[thread_idx, i]:
-                P[thread_idx, i] = D_squared
-                I[thread_idx, i] = i + k
-
-            if ignore_trivial and D_squared < P[thread_idx, i + k]:
-                P[thread_idx, i + k] = D_squared
-                I[thread_idx, i + k] = i
-
-
-@njit(parallel=True, fastmath=True)
-def _scrump(T_A, T_B, m, M_T, Σ_T, μ_Q, σ_Q, orders, orders_ranges, ignore_trivial):
-    """
-    A Numba JIT-compiled version of SCRIMP for parallel computation of the matrix
-    profile and matrix profile indices.
-
-    Parameters
-    ----------
-    T_A : ndarray
-        The time series or sequence for which to compute the matrix profile
-
-    T_B : ndarray
-        The time series or sequence that contain your query subsequences
-        of interest
-
-    m : int
-        Window size
-
-    M_T : ndarray
-        Sliding mean of time series, `T_A`
-
-    Σ_T : ndarray
-        Sliding standard deviation of time series, `T_A`
-
-    μ_Q : ndarray
-        Mean of the query sequence, `Q`, relative to the current sliding window in `T_B`
-
-    σ_Q : ndarray
-        Standard deviation of the query sequence, `Q`, relative to the current
-        sliding window in `T_B`
-
-    orders : ndarray
-        The order of diagonals to process and compute
-
-    orders_ranges : ndarray
-        The start (column 1) and (exclusive) stop (column 2) order indices for
-        each thread
-
-    ignore_trivial : bool
-        Set to `True` if this is a self-join. Otherwise, for AB-join, set this to
-        `False`. Default is `True`.
-
-    Returns
-    -------
-    P : ndarray
-        Matrix profile
-
-    I : ndarray
-        Matrix profile indices
-
-    Notes
-    -----
-    `DOI: 10.1109/ICDM.2018.00099 \
-    <https://www.cs.ucr.edu/~eamonn/SCRIMP_ICDM_camera_ready_updated.pdf>`__
-
-    See Algorithm 1
-    """
-    n = T_B.shape[0]
-    l = n - m + 1
-    n_threads = config.NUMBA_NUM_THREADS
-    P = np.empty((n_threads, l))
-    I = np.empty((n_threads, l), np.int64)
-
-    P[:, :] = np.inf
-    I[:, :] = -1
-
-    for thread_idx in prange(n_threads):
-        # Compute and pdate P, I within a single thread while avoiding race conditions
-        _compute_diagonal(
-            T_A,
-            T_B,
-            m,
-            M_T,
-            Σ_T,
-            μ_Q,
-            σ_Q,
-            orders,
-            orders_ranges[thread_idx, 0],
-            orders_ranges[thread_idx, 1],
-            thread_idx,
-            P,
-            I,
-            ignore_trivial,
-        )
-
-    # Reduction of results from all threads
-    for thread_idx in range(1, n_threads):
-        for i in prange(l):
-            if P[0, i] > P[thread_idx, i]:
-                P[0, i] = P[thread_idx, i]
-                I[0, i] = I[thread_idx, i]
-
-    return np.sqrt(P[0]), I[0]
-
-
 @njit(parallel=True, fastmath=True)
 def _prescrump(
     T_A,
@@ -371,8 +38,8 @@ def _prescrump(
         The time series or sequence for which to compute the matrix profile
 
     T_B : ndarray
-        The time series or sequence that contain your query subsequences
-        of interest
+        The time series or sequence that will be used to annotate T_A. For every
+        subsequence in T_A, its nearest neighbor in T_B will be recorded.
 
     m : int
         Window size
@@ -420,8 +87,8 @@ def _prescrump(
     """
     l = QT.shape[0]
     # Update P[i] relative to all T[j : j + m]
-    Q = T_B[i : i + m]
-    squared_distance_profile[:] = core._mass(Q, T_A, QT, μ_Q[i], σ_Q[i], M_T, Σ_T)
+    Q = T_A[i : i + m]
+    squared_distance_profile[:] = core._mass(Q, T_B, QT, μ_Q[i], σ_Q[i], M_T, Σ_T)
     squared_distance_profile[:] = np.square(squared_distance_profile)
     if excl_zone is not None:
         zone_start = max(0, i - excl_zone)
@@ -439,11 +106,16 @@ def _prescrump(
     for k in range(1, min(s, l - max(i, j))):
         QT_j = (
             QT_j
-            - T_A[i + k - 1] * T_B[j + k - 1]
-            + T_A[i + k + m - 1] * T_B[j + k + m - 1]
+            - T_B[i + k - 1] * T_A[j + k - 1]
+            + T_B[i + k + m - 1] * T_A[j + k + m - 1]
         )
         D_squared = core._calculate_squared_distance(
-            m, QT_j, M_T[i + k], Σ_T[i + k], μ_Q[j + k], σ_Q[j + k],
+            m,
+            QT_j,
+            M_T[i + k],
+            Σ_T[i + k],
+            μ_Q[j + k],
+            σ_Q[j + k],
         )
         if D_squared < P_squared[i + k]:
             P_squared[i + k] = D_squared
@@ -453,9 +125,14 @@ def _prescrump(
             I[j + k] = i + k
     QT_j = QT_j_prime
     for k in range(1, min(s, i + 1, j + 1)):
-        QT_j = QT_j - T_A[i - k + m] * T_B[j - k + m] + T_A[i - k] * T_B[j - k]
+        QT_j = QT_j - T_B[i - k + m] * T_A[j - k + m] + T_B[i - k] * T_A[j - k]
         D_squared = core._calculate_squared_distance(
-            m, QT_j, M_T[i - k], Σ_T[i - k], μ_Q[j - k], σ_Q[j - k],
+            m,
+            QT_j,
+            M_T[i - k],
+            Σ_T[i - k],
+            μ_Q[j - k],
+            σ_Q[j - k],
         )
         if D_squared < P_squared[i - k]:
             P_squared[i - k] = D_squared
@@ -482,8 +159,8 @@ def prescrump(T_A, m, T_B=None, s=None):
         Window size
 
     T_B : ndarray
-        The time series or sequence that contain your query subsequences
-        of interest
+        The time series or sequence that will be used to annotate T_A. For every
+        subsequence in T_A, its nearest neighbor in T_B will be recorded.
 
     s : int
         The sampling interval that defaults to `int(np.ceil(m / 4))`
@@ -503,10 +180,10 @@ def prescrump(T_A, m, T_B=None, s=None):
 
     See Algorithm 2
     """
-    T_A = np.asarray(T_A)
     T_A = T_A.copy()
-    T_A[np.isinf(T_A)] = np.nan
+    T_A = np.asarray(T_A)
     core.check_dtype(T_A)
+    T_A[np.isinf(T_A)] = np.nan
 
     if T_B is None:
         T_B = T_A
@@ -514,34 +191,31 @@ def prescrump(T_A, m, T_B=None, s=None):
     else:
         excl_zone = None
 
-    T_B = np.asarray(T_B)
     T_B = T_B.copy()
-    T_B[np.isinf(T_B)] = np.nan
+    T_B = np.asarray(T_B)
     core.check_dtype(T_B)
+    T_B[np.isinf(T_B)] = np.nan
 
     core.check_window_size(m)
 
-    M_T, Σ_T = core.compute_mean_std(T_A, m)
-    μ_Q, σ_Q = core.compute_mean_std(T_B, m)
+    μ_Q, σ_Q = core.compute_mean_std(T_A, m)
+    M_T, Σ_T = core.compute_mean_std(T_B, m)
 
     T_A[np.isnan(T_A)] = 0
     T_B[np.isnan(T_B)] = 0
 
     n_A = T_A.shape[0]
     n_B = T_B.shape[0]
-    l = n_B - m + 1
-    P_squared = np.empty(l)
-    I = np.empty(l, dtype=np.int64)
-    squared_distance_profile = np.empty(n_A - m + 1)
+    l = n_A - m + 1
+    P_squared = np.full(l, np.inf)
+    I = np.full(l, -1, dtype=np.int64)
+    squared_distance_profile = np.empty(n_B - m + 1)
 
     if s is None:  # pragma: no cover
         s = excl_zone
 
-    P_squared[:] = np.inf
-    I[:] = -1
-
     for i in np.random.permutation(range(0, l, s)):
-        QT = core.sliding_dot_product(T_B[i : i + m], T_A)
+        QT = core.sliding_dot_product(T_A[i : i + m], T_B)
         _prescrump(
             T_A,
             T_B,
@@ -566,11 +240,10 @@ def prescrump(T_A, m, T_B=None, s=None):
 
 class scrump(object):
     """
-    Compute the approximate matrix profile with the parallelized SCRIMP algorthm. For
-    SCRIMP++, set `pre_scrump=True`.
+    Compute an approximate z-normalized matrix profile
 
     This is a convenience wrapper around the Numba JIT-compiled parallelized
-    `_scrump` function which computes the matrix profile according to SCRIMP.
+    `_stump` function which computes the matrix profile according to SCRIMP.
 
     Parameters
     ----------
@@ -578,8 +251,8 @@ class scrump(object):
         The time series or sequence for which to compute the matrix profile
 
     T_B : ndarray
-        The time series or sequence that contain your query subsequences
-        of interest
+        The time series or sequence that will be used to annotate T_A. For every
+        subsequence in T_A, its nearest neighbor in T_B will be recorded.
 
     m : int
         Window size
@@ -594,7 +267,7 @@ class scrump(object):
     pre_scrump : bool
         A flag for whether or not to perform the PreSCRIMP calculation prior to
         computing SCRIMP. If set to `True`, this is equivalent to computing
-        SCRIMP++
+        SCRIMP++ and may lead to faster convergence
 
     s : int
         The size of the PreSCRIMP fixed interval. If `pre-scrump=True` and `s=None`,
@@ -643,8 +316,8 @@ class scrump(object):
             The time series or sequence for which to compute the matrix profile
 
         T_B : ndarray
-            The time series or sequence that contain your query subsequences
-            of interest
+            The time series or sequence that will be used to annotate T_A. For every
+            subsequence in T_A, its nearest neighbor in T_B will be recorded.
 
         m : int
             Window size
@@ -673,8 +346,23 @@ class scrump(object):
             self._ignore_trivial = True
 
         self._m = m
-        self._T_A, self._M_T, self._Σ_T = core.preprocess(T_A, self._m)
-        self._T_B, self._μ_Q, self._σ_Q = core.preprocess(T_B, self._m)
+        (
+            self._T_A,
+            self._μ_Q,
+            self._σ_Q_inverse,
+            self._μ_Q_m_1,
+            self._T_A_subseq_isfinite,
+            self._T_A_subseq_isconstant,
+        ) = core.preprocess_diagonal(T_A, self._m)
+
+        (
+            self._T_B,
+            self._M_T,
+            self._Σ_T_inverse,
+            self._M_T_m_1,
+            self._T_B_subseq_isfinite,
+            self._T_B_subseq_isconstant,
+        ) = core.preprocess_diagonal(T_B, self._m)
 
         if self._T_A.ndim != 1:  # pragma: no cover
             raise ValueError(
@@ -688,9 +376,6 @@ class scrump(object):
                 "For multidimensional STUMP use `stumpy.mstump` or `stumpy.mstumped`"
             )
 
-        core.check_dtype(self._T_A)
-        core.check_dtype(self._T_B)
-
         core.check_window_size(self._m)
 
         if self._ignore_trivial is False and core.are_arrays_equal(
@@ -708,12 +393,12 @@ class scrump(object):
 
         self._n_A = self._T_A.shape[0]
         self._n_B = self._T_B.shape[0]
-        self._l = self._n_B - self._m + 1
+        self._l = self._n_A - self._m + 1
 
-        self._P = np.empty(self._l, dtype=np.float64)
-        self._I = np.empty(self._l, dtype=np.int64)
-        self._P[:] = np.inf
-        self._I[:] = -1
+        self._P = np.empty((self._l, 3), dtype=np.float64)
+        self._I = np.empty((self._l, 3), dtype=np.int64)
+        self._P[:, :] = np.inf
+        self._I[:, :] = -1
 
         self._excl_zone = int(np.ceil(self._m / 4))
 
@@ -726,25 +411,30 @@ class scrump(object):
             else:
                 P, I = prescrump(T_A, m, T_B=T_B, s=s)
             for i in range(P.shape[0]):
-                if self._P[i] > P[i]:
-                    self._P[i] = P[i]
-                    self._I[i] = I[i]
+                if self._P[i, 0] > P[i]:
+                    self._P[i, 0] = P[i]
+                    self._I[i, 0] = I[i]
 
         if self._ignore_trivial:
-            self._orders = np.random.permutation(
-                range(self._excl_zone + 1, self._n_B - self._m + 1)
+            self._diags = np.random.permutation(
+                range(self._excl_zone + 1, self._n_A - self._m + 1)
             )
         else:
-            self._orders = np.random.permutation(
-                range(-(self._n_B - self._m + 1) + 1, self._n_A - self._m + 1)
+            self._diags = np.random.permutation(
+                range(-(self._n_A - self._m + 1) + 1, self._n_B - self._m + 1)
             )
 
         self._n_threads = config.NUMBA_NUM_THREADS
         self._percentage = min(percentage, 1.0)
         self._percentage = max(percentage, 0.0)
         self._n_chunks = int(np.ceil(1.0 / percentage))
-        self._chunk = 1
-        self._start = 0
+        self._ndist_counts = core._count_diagonal_ndist(
+            self._diags, self._m, self._n_A, self._n_B
+        )
+        self._chunk_diags_ranges = core._get_array_ranges(
+            self._ndist_counts, self._n_chunks
+        )
+        self._chunk_idx = 0
 
     def update(self):
         """
@@ -752,49 +442,67 @@ class scrump(object):
         additional new distances (limited by `percentage`) that make up the full
         distance matrix.
         """
-        if self._chunk <= self._n_chunks:
-            orders_ranges = _get_orders_ranges(
-                self._n_threads,
-                self._m,
-                self._n_A,
-                self._n_B,
-                self._orders,
-                self._start,
-                self._percentage,
-            )
+        if self._chunk_idx < self._n_chunks:
+            start_idx, stop_idx = self._chunk_diags_ranges[self._chunk_idx]
 
-            P, I = _scrump(
+            P, I = _stump(
                 self._T_A,
                 self._T_B,
                 self._m,
                 self._M_T,
-                self._Σ_T,
                 self._μ_Q,
-                self._σ_Q,
-                self._orders,
-                orders_ranges,
+                self._Σ_T_inverse,
+                self._σ_Q_inverse,
+                self._M_T_m_1,
+                self._μ_Q_m_1,
+                self._T_A_subseq_isfinite,
+                self._T_B_subseq_isfinite,
+                self._T_A_subseq_isconstant,
+                self._T_B_subseq_isconstant,
+                self._diags[start_idx:stop_idx],
                 self._ignore_trivial,
             )
-            self._start = orders_ranges[:, 1].max()
 
             # Update matrix profile and indices
             for i in range(self._P.shape[0]):
-                if self._P[i] > P[i]:
-                    self._P[i] = P[i]
-                    self._I[i] = I[i]
+                if self._P[i, 0] > P[i, 0]:
+                    self._P[i, 0] = P[i, 0]
+                    self._I[i, 0] = I[i, 0]
+                # left matrix profile and left matrix profile indices
+                if self._P[i, 1] > P[i, 1]:
+                    self._P[i, 1] = P[i, 1]
+                    self._I[i, 1] = I[i, 1]
+                # right matrix profile and right matrix profile indices
+                if self._P[i, 2] > P[i, 2]:
+                    self._P[i, 2] = P[i, 2]
+                    self._I[i, 2] = I[i, 2]
 
-            self._chunk += 1
+            self._chunk_idx += 1
 
     @property
     def P_(self):
         """
         Get the updated matrix profile
         """
-        return self._P.astype(np.float)
+        return self._P[:, 0].astype(np.float)
 
     @property
     def I_(self):
         """
         Get the updated matrix profile indices
         """
-        return self._I.astype(np.int)
+        return self._I[:, 0].astype(np.int)
+
+    @property
+    def left_I_(self):
+        """
+        Get the updated left matrix profile indices
+        """
+        return self._I[:, 1].astype(np.int)
+
+    @property
+    def right_I_(self):
+        """
+        Get the updated right matrix profile indices
+        """
+        return self._I[:, 2].astype(np.int)
